# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/91_tgrd.ipynb.

# %% auto 0
__all__ = ['T', 'P', 'TypeLike', 'GuardFunc', 'istypeq', 'isinst', 'isinstq', 'isof']

# %% ../nbs/91_tgrd.ipynb 4
from types import NoneType
from typing import Type, Tuple, Union, Callable, TypeAlias, TypeGuard, TypeVar, ParamSpec
T = TypeVar('T')
P = ParamSpec('P')

# %% ../nbs/91_tgrd.ipynb 6
TypeLike:  TypeAlias = Union[Type, TypeAlias, Type[T]]
GuardFunc: TypeAlias = Callable[P, TypeGuard[TypeLike]]

# %% ../nbs/91_tgrd.ipynb 7
def istypeq(x: T, t: TypeLike) -> TypeGuard[Union[T, NoneType]]:
    '''Check if `x` is of type `t` or `NoneType`'''
    return isinstance(x, (t, NoneType))

def isinst(x: T, t: TypeLike) -> TypeGuard[T]:
    '''Alias for `isinstance`'''
    return isinstance(x, t)

def isinstq(x: T, t: TypeLike) -> TypeGuard[Union[T, NoneType]]:
    '''Alias for `isinstance`'''
    return isinstance(x, (t, NoneType))

def isof(x: T, t: Tuple[TypeLike, ...]) -> TypeGuard[TypeLike]:
    '''Alias for `isinstance`'''
    return isinst(x, t)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/06_issp.ipynb.

# %% auto 0
__all__ = ['sparr', 'spmtx', 'issparray', 'isspmatrix', 'issparr', 'isspmtx', 'issparse', 'isspr']

# %% ../nbs/06_issp.ipynb 4
from typing import TypeGuard, ForwardRef, TypeAlias, Union
try: from scipy.sparse import sparray, spmatrix
except ImportError: sparray, spmatrix = list, list
sparr: TypeAlias = ForwardRef('sp.sparse.sparray',  module='scipy.sparse', is_class=True)
spmtx: TypeAlias = ForwardRef('sp.sparse.spmatrix', module='scipy.sparse', is_class=True)

# %% ../nbs/06_issp.ipynb 6
def issparray(x) -> TypeGuard[sparr]:
    '''Check if `x` is an `sp.sparse.sparray`.
    
    See Also
    --------
    issparr : Alias for `issparray`.'''
    return isinstance(x, sparray)

def isspmatrix(x) -> TypeGuard[spmtx]:
    '''Check if `x` is an `sp.sparse.spmatrix`.
    
    See Also
    --------
    isspmtx : Alias for `isspmatrix`.
    '''
    return isinstance(x, spmatrix)

def issparr(x) -> TypeGuard[sparr]:
    '''Check if `x` is an `sp.sparse.sparray`.
    
    See Also
    --------
    issparray : Alias for `issparr`.
    '''
    return issparray(x)

def isspmtx(x) -> TypeGuard[spmtx]:
    '''Check if `x` is an `sp.sparse.spmatrix`.
    
    See Also
    --------
    isspmatrix : Alias for `isspmtx`.
    '''
    return isspmatrix(x)

def issparse(x) -> TypeGuard[Union[sparr, spmtx]]:
    '''Check if `x` is an `sp.sparse.sparray` or `sp.sparse.spmatrix`.
    
    See Also
    --------
    issparray : Alias for `issparr`.
    isspmatrix : Alias for `isspmtx`.
    '''
    return issparray(x) or isspmatrix(x)

def isspr(x) -> TypeGuard[Union[sparr, spmtx]]:
    '''Check if `x` is an `sp.sparse.sparray` or `sp.sparse.spmatrix`.
    
    See Also
    --------
    issparray : Alias for `issparr`.
    isspmatrix : Alias for `isspmtx`.
    issparse : Alias for `issparse`.
    '''
    return issparray(x) or isspmatrix(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/99_misc.ipynb.

# %% auto 0
__all__ = ['isin', 'arein']

# %% ../nbs/99_misc.ipynb 4
from typing import Any, Type, Union, Callable, Literal, Iterable, TypeAlias, TypeGuard, ParamSpec

from .core import isiter

# %% ../nbs/99_misc.ipynb 6
def isin(x, refs: Iterable) -> bool: return x in refs
def arein(x: Iterable, refs: Iterable) -> bool:
    return isiter(x) and isiter(refs) and all(isin(y, refs) for y in x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/07_path.ipynb.

# %% auto 0
__all__ = ['ispath', 'ispathlike', 'isdir', 'isfile', 'ishidden']

# %% ../nbs/07_path.ipynb 4
import os, pathlib
from typing import TypeGuard, Union

# %% ../nbs/07_path.ipynb 6
def ispath(x) -> TypeGuard[pathlib.Path]:
    '''Check if `x` is `pathlib.Path`.'''
    return isinstance(x, pathlib.Path)

def ispathlike(x) -> TypeGuard[Union[str, pathlib.Path, os.PathLike]]:
    '''Check if `x` is `str`, `pathlib.Path` or `os.PathLike`.'''
    return isinstance(x, (str, pathlib.Path, os.PathLike))

def isdir(x) -> TypeGuard[pathlib.Path]: 
    '''Check if `x` is `pathlib.Path` and exists.'''
    return ispathlike(x) and pathlib.Path(x).is_dir()

def isfile(x) -> TypeGuard[pathlib.Path]: 
    '''Check if `x` is `pathlib.Path` and exists.'''
    return ispathlike(x) and pathlib.Path(x).is_file()

def ishidden(x) -> TypeGuard[pathlib.Path]: 
    '''Check if `x` is `pathlib.Path` and is a hidden file or directory.'''
    pname = pathlib.Path(x).name
    return ispathlike(x) and (pname.startswith('.') or pname.startswith('_'))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/06_ispd.ipynb.

# %% auto 0
__all__ = ['dataframe', 'pdindex', 'pdseries', 'isdf', 'isidx', 'isseries']

# %% ../nbs/06_ispd.ipynb 4
from typing import TypeGuard, ForwardRef, TypeAlias
try: from pandas import DataFrame, Index, Series
except ImportError: DataFrame, Index, Series = list, list, list
dataframe: TypeAlias = ForwardRef('pd.DataFrame', module='pandas', is_class=True)
pdindex: TypeAlias   = ForwardRef('pd.Index',     module='pandas', is_class=True)
pdseries: TypeAlias  = ForwardRef('pd.Series',    module='pandas', is_class=True)

# %% ../nbs/06_ispd.ipynb 6
def isdf(x) -> TypeGuard[dataframe]:
    '''Check if `x` is an `pd.DataFrame`.'''
    return isinstance(x, DataFrame)

def isidx(x) -> TypeGuard[pdindex]:
    '''Check if `x` is an `pd.Index`.'''
    return isinstance(x, Index)

def isseries(x) -> TypeGuard[pdseries]:
    '''Check if `x` is an `pd.Series`.'''
    return isinstance(x, Series)



__version__ = "0.0.15"
# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00__init__.ipynb.

# %% auto 0
__all__ = []

# %% ../nbs/00__init__.ipynb 3
from itertools import chain
from . import core, comp, insp, dcls, strs, isnp, ispd, issp, path, trch, bool as bmod, isan, wrap, tgrd, misc, alls

from .core import *
from .comp import *
from .insp import *
from .dcls import *
from .strs import *
from .isnp import *
from .ispd import *
from .issp import *
from .path import *
from .trch import *
from .bool import *
from .isan import *
from .wrap import *
from .tgrd import *
from .misc import *
from .alls import *

# %% ../nbs/00__init__.ipynb 4
__all__ = sorted(set(chain(*[
    mod.__all__ for mod in (core, comp, insp, dcls, strs, isnp, ispd, issp, path, trch, bmod, isan, wrap, tgrd, misc, alls, )
])))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/08_trch.ipynb.

# %% auto 0
__all__ = ['pttensor', 'ptdevice', 'isdevice', 'istensor', 'istrc', 'istens', 'iscpu', 'ismps', 'iscuda', 'iscputensor',
           'ismpstensor']

# %% ../nbs/08_trch.ipynb 4
from typing import TypeGuard

# %% ../nbs/08_trch.ipynb 5
from typing import TypeGuard, Literal, Union, ForwardRef, TypeAlias, Iterable
from .core import isstr, isnone
try:
    from torch import Tensor , device    
except ImportError:
    # Define a placeholder type if torch is not installed
    Tensor = list
    device = str

pttensor: TypeAlias = ForwardRef('torch.Tensor', module='torch', is_class=True)
ptdevice: TypeAlias = ForwardRef('torch.device', module='torch', is_class=True)

# %% ../nbs/08_trch.ipynb 8
def isdevice(x) -> TypeGuard[ptdevice]:
    '''Check if `x` is a `torch.device`'''
    return not isnone(x) and ((not isstr(x) and hasattr(x, 'type')) or isinstance(x, device))

def istensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor`
    
    See Also
    --------
    istrc : Alias for `istensor`
    istens : Alias for `istensor`
    '''
    return not isnone(x) and isinstance(x, Tensor) and hasattr(x, 'device')

def istrc(x) -> TypeGuard[ptdevice]:
    '''Check if `x` is a `torch.Tensor` with a `torch.device`.
    
    See Also
    --------
    istensor : Alias for `istrc`
    '''
    return istensor(x)

def istens(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` with a `torch.device`.
    
    See Also
    --------
    istensor : Check if `x` is a `torch.Tensor` with a `torch.device`
    istrc : Alias for `istensor`
    '''
    return istensor(x)



# %% ../nbs/08_trch.ipynb 10
def iscpu(s: str) -> TypeGuard[Literal['cpu']]:
    '''Check if `s` is `'cpu'`'''
    return s == 'cpu'

def ismps(s: str) -> TypeGuard[Literal['mps']]:
    '''Check if `s` is `'mps'`'''
    return s == 'mps'

def iscuda(s: str) -> TypeGuard[Literal['cuda']]:
    '''Check if `s` is `'cuda'`'''
    return s == 'cuda'

# %% ../nbs/08_trch.ipynb 12
def iscputensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` on cpu'''
    return istensor(x) and iscpu(x.device.type)

def ismpstensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` on mps'''
    return istensor(x) and ismps(x.device.type)

def ismpstensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` on cuda'''
    return istensor(x) and iscuda(x.device.type)
